package mitarbeiterDao;

import fachklassen.Mitarbeiter;
import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.filter.Filters;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

public class XmlMitarbeiterDAO implements IMitarbeiterDAO {

    private Document xmlDocument;
    private String fileName;
    private Element mitarbeiterVerwaltung;

    public XmlMitarbeiterDAO() throws MitarbeiterException {
        this("employees.xml");
    }

    public XmlMitarbeiterDAO(String fileName) throws MitarbeiterException {
        try {
            this.fileName = fileName;
            SAXBuilder builder = new SAXBuilder();
            xmlDocument = builder.build(new File(fileName));
            mitarbeiterVerwaltung = xmlDocument.getRootElement();
        } catch (Exception e) {
            // e.printStackTrace();
            throw new MitarbeiterException("XmlMitarbeiterDAO.XmlMitarbeiterDAO");
        }
    }

    @Override
    public boolean mitarbeiterExists(int id) {
        XPathFactory xpf = XPathFactory.instance();
        XPathExpression<Element> xPathExpression = xpf.compile("//employee[@id='" + id + "']", Filters.element());
        List<Element> mitarbeiterList = xPathExpression.evaluate(xmlDocument);
        return !mitarbeiterList.isEmpty();
    }

    @Override
    public Mitarbeiter getMitarbeiter(int id) throws MitarbeiterException {
        XPathFactory xpf = XPathFactory.instance();
        XPathExpression<Element> xPathExpression = xpf.compile("//employee[@id='" + id + "']", Filters.element());
        List<Element> mitarbeiterList = xPathExpression.evaluate(xmlDocument);

        if (mitarbeiterList.size() > 0) {
            Element e = mitarbeiterList.get(0);
            return new Mitarbeiter(Integer.parseInt(e.getAttributeValue("id")), 
                    e.getChildText("name"),e.getChildText("gender"), 
                    Integer.parseInt(e.getChildText("age")), e.getChildText("role"));
        } else {
            return null;
        }
    }

    @Override
    public Mitarbeiter[] getAllMitarbeiter() throws MitarbeiterException {
        ArrayList<Mitarbeiter> alleMitarbeiter = new ArrayList<>();
        Mitarbeiter[] temp = new Mitarbeiter[1];
        XPathFactory xpf = XPathFactory.instance();
        XPathExpression<Element> xPathExpression = xpf.compile("//employee", Filters.element());
        List<Element> mitarbeiterList = xPathExpression.evaluate(xmlDocument);
        if (mitarbeiterList.size() > 0) {
            for (Element e : mitarbeiterList) {
                alleMitarbeiter.add(new Mitarbeiter(Integer.parseInt(e.getAttributeValue("id")), 
                        e.getChildText("name"), e.getChildText("gender"), 
                        Integer.parseInt(e.getChildText("age")), e.getChildText("role")));
            }
            return alleMitarbeiter.toArray(temp);
        } else {
            return null;
        }
    }

    @Override
    public void addMitarbeiter(Mitarbeiter mitarbeiter) throws MitarbeiterException {
        if (!this.mitarbeiterExists(mitarbeiter.getId())) {
            Element neuerMitarbeiter = new Element("employee");
            neuerMitarbeiter.setAttribute("id", Integer.toString(mitarbeiter.getId()));
            neuerMitarbeiter.addContent(new Element("age").addContent(Integer.toString(mitarbeiter.getAlter())));
            neuerMitarbeiter.addContent(new Element("name").addContent(mitarbeiter.getName()));
            neuerMitarbeiter.addContent(new Element("gender").addContent(mitarbeiter.getGeschlecht()));
            neuerMitarbeiter.addContent(new Element("role").addContent(mitarbeiter.getFunktion()));
            mitarbeiterVerwaltung.addContent(neuerMitarbeiter);
            this.saveFile();
        }
    }

    @Override
    public void changeMitarbeiter(Mitarbeiter mitarbeiter) throws MitarbeiterException {
        if (this.mitarbeiterExists(mitarbeiter.getId())) {
            this.removeMitarbeiter(mitarbeiter);
            this.addMitarbeiter(mitarbeiter);
        }
    }

    @Override
    public void removeMitarbeiter(Mitarbeiter mitarbeiter) throws MitarbeiterException {
        if (this.mitarbeiterExists(mitarbeiter.getId())) {
            XPathFactory xpf = XPathFactory.instance();
            XPathExpression<Element> xPathExpression = xpf.compile("//employee[@id='" + mitarbeiter.getId() + "']", Filters.element());
            List<Element> mitarbeiterToRemove = xPathExpression.evaluate(xmlDocument);
            for (Element e : mitarbeiterToRemove) {
                mitarbeiterVerwaltung.removeContent(e);
            }
            this.saveFile();
        }
    }

    private void saveFile() throws MitarbeiterException {
        try {
            XMLOutputter outp = new XMLOutputter();
            outp.setFormat(Format.getPrettyFormat());
            outp.output(xmlDocument, new FileOutputStream(new File(fileName)));
        } catch (Exception e) {
            throw new MitarbeiterException("XmlMitarbeiterDAO.saveFile");
        }
    }
}
